h1 { font-family: sans-serif; font-size: x-large;}
h2 { font-family: sans-serif; font-size: large;
     margin-top: .5in;
     border-top-color: gray;
     border-top-style: solid;
     border-top-width: thin}
h3 { font-family: sans-serif; font-size: medium;
     margin-top: .4in}
h4 { font-family: sans-serif;
     margin-top: .3in }
h5 { font-family: serif;
     margin-top: .3in }
body  { margin-left: .2in}
ul { margin-left: .4in}
ol { margin-left: .45in}
li { margin-bottom: 2%}
